"""
This ``urls.py`` is only used when running the tests via ``runtests.py``.
zR you know, every app must be hooked into yout main ``urls.py`` so that
you can actually reach the app's views (provided it has any views, of course).

"""
from django.conf.urls.defaults import include, patterns, url
from django.contrib import admin
from django.http import HttpResponse


admin.autodiscover()


urlpatterns = patterns(
    '',
    url(r'^$', lambda req: HttpResponse('Dummy home view'), name='dummy_home'),
    url(r'^paypal/', include('paypal_express_checkout.urls')),
    url(r'^admin/', include(admin.site.urls)),
)



"""Tests for the forms of the ``paypal_express_checkout`` app."""
from mock import Mock, PropertyMock, patch
from httplib import HTTPException

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import Http404
from django.test import TestCase

from django_libs.tests.factories import UserFactory

from ..forms import (
    DoExpressCheckoutForm,
    PayPalFormMixin,
    SetExpressCheckoutFormMixin,
    SetExpressCheckoutItemForm,
)
from ..models import PurchasedItem
from ..constants import PAYPAL_DEFAULTS
from ..settings import API_URL
from .factories import ItemFactory, PaymentTransactionFactory
from ..settings import LOGIN_URL


class PayPalFormMixinTestCase(TestCase):
    """Tests for the ``PayPalFormMixin`` class."""
    longMessage = True

    def setUp(self):
        super(PayPalFormMixinTestCase, self).setUp()
        self.paypal_response = 'ACK=Success&TOKEN=abc123'

    @patch('paypal_express_checkout.forms.urllib2')
    def test_call_paypal(self, urllib2_mock):
        mixin = PayPalFormMixin()

        response_mock = Mock()
        response_mock.read = Mock(return_value=self.paypal_response)
        urllib2_mock.urlopen.return_value = response_mock
        response = mixin.call_paypal(API_URL, {})
        self.assertEqual(response['ACK'], ['Success'], msg=(
            'Should parse the response from paypal and return it as a dict'))

        with patch.object(mixin, 'log_error') as log_error_mock:
            urllib2_mock.urlopen = PropertyMock(side_effect=HTTPException)
            mixin.call_paypal(API_URL, {})
            self.assertEqual(log_error_mock.call_count, 1, msg=(
                'Should log an error if calling the PayPal API fails.'))


class DoExpressCheckoutFormTestCase(TestCase):
    """Tests for the ``DoExpressCheckoutForm`` form class."""
    longMessage = True

    def setUp(self):
        self.token = 'abc123'
        self.transaction = PaymentTransactionFactory(transaction_id=self.token)
        self.user = self.transaction.user
        self.valid_response = {'ACK': ['Success'],
                               'PAYMENTINFO_0_TRANSACTIONID': [self.token]}
        self.invalid_response = {'ACK': ['Failure']}
        self.valid_data = {'token': self.token, 'PayerID': 'PAYERID123'}

    @patch.object(PayPalFormMixin, 'call_paypal')
    def test_form(self, call_paypal_mock):
        call_paypal_mock.return_value = self.valid_response
        form = DoExpressCheckoutForm(user=self.user, data=self.valid_data)
        self.assertTrue(form.is_valid, msg='The form should be valid.')

        resp = form.do_checkout()
        self.assertEqual(resp['Location'], reverse('paypal_success'))

        call_paypal_mock.return_value = self.invalid_response
        form = DoExpressCheckoutForm(user=self.user, data=self.valid_data)
        self.assertTrue(form.is_valid, msg='The form should be valid.')

        resp = form.do_checkout()
        self.assertEqual(resp['Location'], reverse('paypal_error'))

        self.transaction.delete()
        self.assertRaises(Http404, DoExpressCheckoutForm,
                          **{'user': self.user, 'data': self.valid_data})


class SetExpressCheckoutFormMixinTestCase(TestCase):
    """Tests for the ``SetExpressCheckoutFormMixin`` mixin."""
    longMessage = True
    maxDiff = None

    def setUp(self):
        self.user = UserFactory()
        self.item1 = ItemFactory(name='item1')
        self.item2 = ItemFactory(name='item2')
        self.item_list = [(self.item1, 1), (self.item2, 0)]
        self.expected_post_data = {
            'L_PAYMENTREQUEST_0_NAME0': self.item1.name,
            'L_PAYMENTREQUEST_0_DESC0': self.item1.description,
            'L_PAYMENTREQUEST_0_AMT0': self.item1.value,
            'L_PAYMENTREQUEST_0_QTY0': 1,
            'METHOD': 'SetExpressCheckout',
            'PAYMENTREQUEST_0_AMT': self.item1.value,
            'PAYMENTREQUEST_0_ITEMAMT': self.item1.value,
            'RETURNURL': settings.HOSTNAME + reverse('paypal_confirm'),
            'CANCELURL': settings.HOSTNAME + reverse('paypal_canceled'),
        }
        self.expected_post_data.update(PAYPAL_DEFAULTS.copy())
        self.token = 'abc123'
        self.valid_response = {'ACK': ['Success'], 'TOKEN': [self.token]}
        self.invalid_response = {'ACK': ['Failure']}

    @patch.object(PayPalFormMixin, 'call_paypal')
    def test_mixin(self, call_paypal_mock):
        form = SetExpressCheckoutFormMixin(self.user)
        self.assertRaises(NotImplementedError, form.get_item)
        self.assertRaises(NotImplementedError, form.get_quantity)
        self.assertRaises(NotImplementedError, form.get_items_and_quantities)
        self.assertEqual(form.get_post_data(self.item_list),
                         self.expected_post_data)

        old_item_and_qty = SetExpressCheckoutFormMixin.get_items_and_quantities
        SetExpressCheckoutFormMixin.get_items_and_quantities = Mock(
            return_value=self.item_list)
        form = SetExpressCheckoutFormMixin(self.user)

        call_paypal_mock.return_value = self.valid_response
        resp = form.set_checkout()
        self.assertEqual(resp['Location'], LOGIN_URL + self.token)

        call_paypal_mock.return_value = self.invalid_response
        resp = form.set_checkout()
        self.assertEqual(resp['Location'], reverse('paypal_error'))

        SetExpressCheckoutFormMixin.get_items_and_quantities = old_item_and_qty


class SetExpressCheckoutItemFormTestCase(TestCase):
    """Tests for the ``SetExpressCheckoutItemForm`` form class."""
    longMessage = True

    def setUp(self):
        self.user = UserFactory()
        self.item = ItemFactory()

        self.paypal_response = {
            'ACK': ['Success'],
            'TOKEN': ['abc123'],
        }

        self.expected_response = {
            'USER': settings.PAYPAL_USER,
            'PWD': settings.PAYPAL_PWD,
            'SIGNATURE': settings.PAYPAL_SIGNATURE,
            'VERSION': '91.0',
            'PAYMENTREQUEST_0_PAYMENTACTION': 'Sale',
            'METHOD': 'SetExpressCheckout',

            'L_PAYMENTREQUEST_0_NAME0': self.item.name,
            'L_PAYMENTREQUEST_0_DESC0': self.item.description,
            'L_PAYMENTREQUEST_0_AMT0': self.item.value,
            'L_PAYMENTREQUEST_0_QTY0': 1,

            'PAYMENTREQUEST_0_AMT': self.item.value,
            'PAYMENTREQUEST_0_ITEMAMT': self.item.value,
            'RETURNURL': settings.HOSTNAME + reverse(
                'paypal_confirm'),
            'CANCELURL': settings.HOSTNAME + reverse(
                'paypal_canceled')
        }
        self.data = {
            'item': self.item.pk,
            'quantity': 1,
        }

        self.old_call_paypal = PayPalFormMixin.call_paypal
        PayPalFormMixin.call_paypal = Mock(return_value=self.paypal_response)

    def tearDown(self):
        PayPalFormMixin.call_paypal = self.old_call_paypal

    def test_is_valid_and_calls_paypal(self):
        """
        Test if the ``SetExpressCheckoutItemForm`` validates and calls PayPal.

        """
        form = SetExpressCheckoutItemForm(user=self.user, data=self.data)
        self.assertTrue(form.is_valid(), msg='The form should be valid.')

        resp = form.set_checkout()
        self.assertEqual(resp.status_code, 302, msg=(
            'Response should redirect.'))
        self.assertEqual(
            resp.items()[1][1], settings.PAYPAL_LOGIN_URL + 'abc123', msg=(
                'Should redirect to PayPal Login.'))
        self.assertEqual(PurchasedItem.objects.all().count(), 1, msg=(
            'Should create a PurchasedItem object when saving the'
            ' transaction'))

        self.paypal_response.update({
            'ACK': ['Failure']})
        resp = form.set_checkout()
        self.assertEqual(resp.status_code, 302, msg=(
            'Response should redirect.'))
        self.assertEqual(
            resp.items()[1][1], reverse('paypal_error'),
            msg='Should redirect to PaymentErrorView.')



"""Tests for the models of the ``paypal_express_checkout`` app."""
from django.test import TestCase

from .. import models
from . import factories


class PaymentTransactionTestCase(TestCase):
    """Tests for the ``PaymentTransaction`` model."""
    longMessage = True

    def test_instantiation(self):
        """Testing instantiation of the ``PaymentTransaction`` model."""
        transaction = models.PaymentTransaction()
        self.assertTrue(transaction)


class PaymentTransactionErrorTestCase(TestCase):
    """Tests for the ``PaymentTransactionError`` model."""
    longMessage = True

    def test_instantiation(self):
        """Testing instantiation of the ``PaymentTransactionError`` model."""
        error = models.PaymentTransactionError()
        self.assertTrue(error)


class PurchasedItemTestCase(TestCase):
    """Tests for the ``PurchasedItem`` model."""
    longMessage = True

    def test_model(self):
        instance = factories.PurchasedItemFactory()
        self.assertTrue(instance.pk)



"""The urls for the ``paypal_express_checkout`` app."""
from django.conf.urls.defaults import patterns, url

from paypal_express_checkout.views import (
    DoExpressCheckoutView,
    IPNListenerView,
    PaymentCancelView,
    PaymentErrorView,
    PaymentSuccessView,
    SetExpressCheckoutView,
)


urlpatterns = patterns(
    '',
    url(
        r'^$',
        SetExpressCheckoutView.as_view(),
        name='paypal_checkout'
    ),

    url(
        r'^confirm/$',
        DoExpressCheckoutView.as_view(),
        name='paypal_confirm',
    ),

    url(
        r'^canceled/$',
        PaymentCancelView.as_view(),
        name='paypal_canceled',
    ),

    url(
        r'^error/$',
        PaymentErrorView.as_view(),
        name='paypal_error',
    ),

    url(
        r'^success/$',
        PaymentSuccessView.as_view(),
        name='paypal_success',
    ),

    url(
        r'ipn/$',
        IPNListenerView.as_view(),
        name='ipn_listener',
    ),

)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'PurchasedItem.item'
        db.alter_column('paypal_express_checkout_purchaseditem', 'item_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['paypal_express_checkout.Item'], null=True))

    def backwards(self, orm):

        # User chose to not deal with backwards NULL issues for 'PurchasedItem.item'
        raise RuntimeError("Cannot reverse this migration. 'PurchasedItem.item' and its values cannot be restored.")

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'paypal_express_checkout.item': {
            'Meta': {'object_name': 'Item'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '4000'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '2048'}),
            'value': ('django.db.models.fields.DecimalField', [], {'max_digits': '8', 'decimal_places': '2'})
        },
        'paypal_express_checkout.paymenttransaction': {
            'Meta': {'ordering': "['-date', 'transaction_id']", 'object_name': 'PaymentTransaction'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'transaction_id': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'value': ('django.db.models.fields.DecimalField', [], {'max_digits': '8', 'decimal_places': '2'})
        },
        'paypal_express_checkout.paymenttransactionerror': {
            'Meta': {'object_name': 'PaymentTransactionError'},
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'response': ('django.db.models.fields.TextField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'paypal_express_checkout.purchaseditem': {
            'Meta': {'ordering': "['-transaction__date', 'transaction__transaction_id']", 'object_name': 'PurchasedItem'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['paypal_express_checkout.Item']", 'null': 'True', 'blank': 'True'}),
            'quantity': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'transaction': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['paypal_express_checkout.PaymentTransaction']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['paypal_express_checkout']



import os
from setuptools import setup, find_packages
import paypal_express_checkout


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''


setup(
    name="django-paypal-express-checkout",
    version=paypal_express_checkout.__version__,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    license='The MIT License',
    platforms=['OS Independent'],
    keywords='django, url, paypal, paypal_express_checkout, API',
    author='Daniel Kaufhold',
    author_email='daniel.kaufhold@bitmazk.com',
    url="https://github.com/bitmazk/django-paypal-express-checkout",
    packages=find_packages(),
    include_package_data=True,
    tests_require=[
        'fabric',
        'factory_boy',
        'django-nose',
        'coverage',
        'django-coverage',
        'mock',
        'django-libs'
    ],
    test_suite='paypal_express_checkout.tests.runtests.runtests',
)
